#include <SDL.h>
#include <SDL_mixer.h>

#include "sdl_gp2x.h"
#include "sdl_mixer_gp2x.h"

void InitSDL_mixer(int freq, int channels, int buffer)
{
	// Init mixer API
	printf("Initializing mixer...");
	if(SDL_Init(SDL_INIT_AUDIO) == -1)
	{
		printf("\t\t\t\tERROR\n");
		SDL_Quit();
	}
	else
		printf("\t\t\t\tOK\n");

	// Set audio settings
	printf("Audio settings\n(%ihz, %i channels, %i buffer)...", freq, channels, buffer);
	if(Mix_OpenAudio(freq, AUDIO_S16, channels, buffer) < 0)
	{
		printf("\t\tERROR\n");
		Mix_CloseAudio();
		SDL_Quit();
	}
	else
		printf("\t\tOK\n");
}

void QuitSDL_mixer()
{
	// Quit SDL_mixer
	Mix_CloseAudio();
}

void AllocateChannels(int channels)
{
	// Allocating mixing channels
	printf("Allocating %i mixing channels...", channels);
	if(Mix_AllocateChannels(channels) < 0) printf("\t\t\tERROR\n"); else printf("\t\t\tOK\n");
}

Mix_Chunk *LoadChannel(const char *c)
{
	Mix_Chunk *chunk;

	// Load WAV file
	printf("Loading \"%s\"...", c);
	chunk = Mix_LoadWAV(c);
	if(!chunk) printf("\t\t\t\tERROR\n"); else printf("\t\t\t\tOK\n");

	// Return chunk
	return chunk;
}

Mix_Music *LoadMusic(const char *c)
{
	Mix_Music *music;

	// Load music file
	printf("Loading \"%s\"...", c);
	music = Mix_LoadMUS(c);
	if(!music) printf("\t\t\t\tERROR\n"); else printf("\t\t\t\tOK\n");

	// Return music
	return music;
}

void CloseChannel(Mix_Chunk *chunk)
{
	// Close channel
	Mix_FreeChunk(chunk);
}

void CloseMusic(Mix_Music *music)
{
	// Close music
	Mix_FreeMusic(music);
}

char *GetMusicFormat(Mix_Music *music)
{
	char *format;

	// Get music format
	switch( Mix_GetMusicType(music) )
	{
		case MUS_CMD:
			format = "Command based format";
			break;
		case MUS_WAV:
			format = "WAVE/RIFF format";
			break;
		case MUS_MOD:
			format = "MOD format";
			break;
		case MUS_MID:
			format = "MIDI format";
			break;
		case MUS_OGG:
			format = "OGG format";
			break;
		case MUS_MP3:
			format = "MP3 format";
			break;
		default:
			format = "Unknown format";
			break;
	}

	// Return format
	return format;
}

void SetChannelVolume(int channel, int volume)
{
	// Set channel volume
	Mix_Volume(channel, volume);
}

void SetMusicVolume(int volume)
{
	// Set music volume
	Mix_VolumeMusic(volume);
}

void PauseChannel(int channel)
{
	// Pause channel
	Mix_Pause(channel);
}

void PauseMusic()
{
	// If music is playing, pause music, else, resume music
	if(Mix_PlayingMusic()) Mix_PauseMusic(); else Mix_ResumeMusic();
}

void StopChannel(int channel)
{
	// Stop channel
	Mix_HaltChannel(channel);
}

void StopMusic()
{
	// If music is playing, stop music
	if(Mix_PlayingMusic()) Mix_HaltMusic();
}

void SetMusicPosition(double pos)
{
	// Set music position
	Mix_SetMusicPosition(pos);
}

void PlayChannel(Mix_Chunk *chunk, int channel, int loops)
{
	// Play channel
	if(Mix_PlayChannel(channel, chunk, loops) < 0) printf("Error playing channel.\n");
}

void PlayChannelTimed(Mix_Chunk *chunk, int channel, int loops, int ticks)
{
	// Play channel during indicated time
	if(Mix_PlayChannelTimed(channel, chunk, loops, ticks) < 0) printf("Error playing channel.\n");
}

void FadeInChannel(Mix_Chunk *chunk, int channel, int loops, int ms)
{
	// Play channel with fade in effect
	if(Mix_FadeInChannel(channel, chunk, loops, ms) < 0) printf("Error playing channel.\n");
}

void FadeInChannelTimed(Mix_Chunk *chunk, int channel, int loops, int ms, int ticks)
{
	// Play channel during indicated time with fade in effect
	if(Mix_FadeInChannelTimed(channel, chunk, loops, ms, ticks) < 0) printf("Error playing channel.\n");
}

void FadeOutChannel(int channel, int ms)
{
	// Stop channel with fade out effect
	Mix_FadeOutChannel(channel, ms);
}

void PlayMusic(Mix_Music *music, int loops)
{
	char *format;

	// If music is not playing
	if(!Mix_PlayingMusic())
	{
		// Get music format
		format = GetMusicFormat(music);

		// Play music
		printf("Playing music (%s)...", format);
		if(Mix_PlayMusic(music, loops) < 0) printf("\t\t\tERROR\n"); else printf("\t\t\tOK\n");
	}
}

void FadeInMusic(Mix_Music *music, int loops, int ms)
{
	char *format;

	// If music is not playing
	if(!Mix_PlayingMusic())
	{
		// Get music format
		format = GetMusicFormat(music);

		// Play music with fade in effect
		printf("Playing music (%s)...", format);
		if(Mix_FadeInMusic(music, loops, ms) < 0) printf("\t\t\tERROR\n"); else printf("\t\t\tOK\n");
	}
}

void FadeInMusicPos(Mix_Music *music, int loops, int ms, double pos)
{
	char *format;

	// If music is not playing
	if(!Mix_PlayingMusic())
	{
		// Get music format
		format = GetMusicFormat(music);

		// Play music from a position with fade in effect
		printf("Playing music (%s)...", format);
		if(Mix_FadeInMusicPos(music, loops, ms, pos) < 0) printf("\t\t\tERROR\n"); else printf("\t\t\tOK\n");
	}
}

void FadeOutMusic(int ms)
{
	// Stop music with fade out effect
	if(!Mix_FadeOutMusic(ms) && Mix_PlayingMusic()) SDL_Delay(100);
}
